from argparse import ArgumentParser
from collections import defaultdict
from pathlib import Path
from numpy import nan
import pandas as pd
import yaml

from parser import parse_mother_request


df = None

def _get_parser() -> ArgumentParser:
    parser = ArgumentParser()
    parser.add_argument(
        "path",
        help="Path for workflow. Serves as input and output.",
        type=str,
    )
    return parser


def _row_to_dict(row: pd.Series, prefix: str = ""):
    # Pandas row to dict
    dct = row.to_dict()
    # Remove unused fields
    dct = {key: value for key, value in dct.items() if value is not nan}
    # Remove prefix from keys
    if prefix:
        dct = {key.removeprefix(prefix): value for key, value in dct.items()}
    return dct


def create_yaml_dicts(path: Path):
    global df  # pylint: disable=W0603

    mother_request = path / "conf" / "mother_request.yml"
    df = parse_mother_request(mother_request)
    opa_columns = [col for col in df.columns if col.startswith("opa_")]
    gsv_columns = [col for col in df.columns if col.startswith("gsv_")]

    dcts = defaultdict(dict)
    for app, app_df in df.groupby("Application"):
        yaml_name = f"{app.lower()}.yml"
        for i, (_, request) in enumerate(app_df.iterrows(), start=1):
            dcts[yaml_name][i] = {
                    "GSVREQUEST": _row_to_dict(request[gsv_columns], "gsv_"),
                    "OPAREQUEST": _row_to_dict(request[opa_columns], "opa_"),
            }

    return dcts


def write_yamls(path: Path):
    yamls = create_yaml_dicts(path)
    yamls_path = path / "conf" / "applications"
    for yaml_filename, yaml_content in yamls.items():
        yaml_filepath = yamls_path / yaml_filename
        with open(yaml_filepath, "w", encoding="utf-8") as wf:
            yaml.safe_dump(yaml_content, wf, default_flow_style=False)


if __name__ == "__main__":
    args = _get_parser().parse_args()
    write_yamls(Path(args.path))
